import { useState, useEffect, useRef } from 'react';
import { useGame } from '../context/GameContext';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faPlus, faTrash, faCheck, faMusic, faVideoCamera, faSearch, faSpinner, faExternalLinkAlt, faTimes } from '@fortawesome/free-solid-svg-icons';

const SongSubmissionScreen = () => {
  const { lobby, currentPlayer, addSong, removeSong, setPlayerReady, searchYouTube, getYouTubeMetadata } = useGame();
  const [songs, setSongs] = useState([]);
  const [isReady, setIsReady] = useState(false);
  const [songForm, setSongForm] = useState({
    youtubeLink: ''
  });
  const [isFormVisible, setIsFormVisible] = useState(false);
  const [searchQuery, setSearchQuery] = useState('');
  const [searchResults, setSearchResults] = useState([]);
  const [isSearching, setIsSearching] = useState(false);
  const [isLoadingMetadata, setIsLoadingMetadata] = useState(false);
  const [selectedVideo, setSelectedVideo] = useState(null);
  const [previewVisible, setPreviewVisible] = useState(false);
  const searchTimeout = useRef(null);
  
  useEffect(() => {
    if (lobby) {
      // Find current player's songs
      const player = lobby.players.find(p => p.id === currentPlayer.id);
      if (player) {
        setIsReady(player.isReady);
      }
    }
  }, [lobby, currentPlayer]);

  // Get player's songs from the server
  useEffect(() => {
    const fetchPlayerSongs = async () => {
      if (lobby && currentPlayer) {
        console.log('Fetching songs for player:', currentPlayer);
        console.log('All players in lobby:', lobby.players.map(p => ({ id: p.id, name: p.name })));
        
        // Find the current player by their ID, name, or socket ID
        let player = lobby.players.find(p => p.id === currentPlayer.id);
        
        // If not found by ID, try by name as fallback
        if (!player) {
          player = lobby.players.find(p => p.name === currentPlayer.name);
          console.log('Player not found by ID, trying by name. Found:', player);
        }
        
        // If player found and has songs, update the state
        if (player) {
          console.log('Found player:', player);
          if (player.songs && Array.isArray(player.songs)) {
            console.log('Found player songs for', player.name, ':', player.songs.length);
            console.log('Songs data:', player.songs);
            setSongs(player.songs || []);
          } else {
            console.log('No songs array for player:', player);
            setSongs([]);
          }
        } else {
          console.error('Player not found in lobby! Current player:', currentPlayer.id);
          console.log('Available players:', lobby.players.map(p => p.id));
          setSongs([]);
        }
      }
    };
    
    fetchPlayerSongs();
    // Important: This effect should run whenever the lobby state changes
  }, [lobby, currentPlayer]);

  // Extract video ID from YouTube URL
  const extractVideoId = (url) => {
    if (!url) return null;
    
    const patterns = [
      /(?:https?:\/\/)?(?:www\.)?youtube\.com\/watch\?v=([^&]+)/,
      /(?:https?:\/\/)?(?:www\.)?youtube\.com\/embed\/([^/?]+)/,
      /(?:https?:\/\/)?(?:www\.)?youtu\.be\/([^/?]+)/
    ];
    
    for (const pattern of patterns) {
      const match = url.match(pattern);
      if (match && match[1]) {
        return match[1];
      }
    }
    
    return null;
  };

  // Debounced search function
  const handleSearch = async (query) => {
    if (!query.trim()) {
      setSearchResults([]);
      return;
    }
    
    setIsSearching(true);
    try {
      const results = await searchYouTube(query);
      setSearchResults(results || []);
    } catch (error) {
      console.error('Search failed:', error);
    } finally {
      setIsSearching(false);
    }
  };

  const handleInputChange = async (e) => {
    const { name, value } = e.target;
    
    if (name === 'searchQuery') {
      setSearchQuery(value);
      
      // Check if the input might be a YouTube link
      const videoId = extractVideoId(value);
      if (videoId) {
        setSongForm({ youtubeLink: value });
        
        // Set basic information immediately for a responsive UI
        const basicVideoInfo = {
          id: videoId,
          url: value,
          thumbnail: `https://img.youtube.com/vi/${videoId}/mqdefault.jpg`
        };
        setSelectedVideo(basicVideoInfo);
        setSearchResults([]); // Clear any existing search results
        
        // Fetch additional metadata from the server
        try {
          setIsLoadingMetadata(true);
          const metadata = await getYouTubeMetadata(videoId);
          if (metadata) {
            // Update selected video with full metadata
            setSelectedVideo({
              ...basicVideoInfo,
              title: metadata.title || 'Unknown Title',
              artist: metadata.artist || 'Unknown Artist',
              thumbnail: metadata.thumbnail || basicVideoInfo.thumbnail
            });
          }
        } catch (error) {
          console.error('Error fetching video metadata:', error);
        } finally {
          setIsLoadingMetadata(false);
        }
      } else if (value.trim()) {
        // Clear any previous timeout
        if (searchTimeout.current) {
          clearTimeout(searchTimeout.current);
        }
        
        // Set a new timeout to prevent too many API calls
        searchTimeout.current = setTimeout(() => handleSearch(value), 500);
      } else {
        setSearchResults([]);
        setSelectedVideo(null);
      }
    }
  };
  
  // Function to toggle video preview
  const togglePreview = (result) => {
    if (selectedVideo && selectedVideo.id === result.id) {
      setPreviewVisible(!previewVisible);
    } else {
      setSelectedVideo(result);
      setPreviewVisible(true);
    }
  };

  const handleSelectSearchResult = (result) => {
    // Make sure we have a complete result with all required fields
    const completeResult = {
      id: result.id,
      url: result.url || `https://www.youtube.com/watch?v=${result.id}`,
      title: result.title || 'Unknown Title',
      artist: result.artist || 'Unknown Artist',
      thumbnail: result.thumbnail || `https://img.youtube.com/vi/${result.id}/mqdefault.jpg`
    };
    
    // When a search result is selected, store the YouTube URL
    setSongForm({
      youtubeLink: completeResult.url
    });
    
    // Store the selected video with all necessary data for submission
    setSelectedVideo(completeResult);
    
    // Keep the search results visible but update the query field
    setSearchQuery(completeResult.title);
  };

  const handleAddSong = (e) => {
    e.preventDefault();
    
    // Use selected video data if available, otherwise fallback to search query or direct input
    let songData;
    
    // Generate a consistent ID format that will work for deletion later
    const generateSongId = () => `song_${Date.now()}_${Math.random().toString(36).substr(2, 9)}`;
    
    if (selectedVideo) {
      // We have a selected video with full details - use all available metadata
      songData = {
        youtubeLink: selectedVideo.url,
        title: selectedVideo.title,
        artist: selectedVideo.artist,
        thumbnail: selectedVideo.thumbnail,
        id: generateSongId() // Consistent ID format
      };
      
      console.log("Adding song with full metadata:", songData);
    } else {
      // Extract YouTube URL from search query or direct input
      const youtubeLink = searchQuery.trim() || songForm.youtubeLink.trim();
      if (!youtubeLink) return;
      
      // Extract video ID to check if it's a valid YouTube link
      const videoId = extractVideoId(youtubeLink);
      if (videoId) {
        // It's a YouTube link, send it to the server for metadata resolution
        songData = { 
          youtubeLink: youtubeLink,
          // Include the videoId to make server-side processing easier
          videoId: videoId,
          id: generateSongId() // Consistent ID format
        };
        
        console.log("Adding song with YouTube link:", songData);
      } else {
        // Not a YouTube link - treat as a search query
        alert("Please enter a valid YouTube link or select a search result");
        return;
      }
    }
    
  // Add the song and update UI when the server responds
  // We use the Promise-based approach to wait for the server-generated ID for proper deletion
  
  // Add the song with callback to update songs when the server responds
  addSong(songData)
    .then((updatedLobby) => {
      console.log('Song added successfully, received updated lobby:', updatedLobby);
      if (updatedLobby && currentPlayer) {
        const player = updatedLobby.players.find(p => p.id === currentPlayer.id);
        if (player && player.songs) {
          console.log('Setting songs from addSong response:', player.songs);
          setSongs([...player.songs]); // Create a new array to ensure state update
        } else {
          console.warn('Player or songs not found in updated lobby');
        }
      } else {
        console.warn('Missing lobby or currentPlayer in response');
      }
    })
    .catch(error => {
      console.error('Error adding song:', error);
    });
  
  // Reset form
  setSongForm({ youtubeLink: '' });
  setSearchQuery('');
  setSearchResults([]);
  setSelectedVideo(null);
  setIsFormVisible(false);
  };

  const handleRemoveSong = (songId) => {
    removeSong(songId);
  };

  const handleSetReady = () => {
    if (songs.length === lobby.settings.songsPerPlayer) {
      setPlayerReady();
      setIsReady(true);
    }
  };

  // Check if we've submitted enough songs
  const canSubmitMoreSongs = lobby && songs.length < lobby.settings.songsPerPlayer;

  // Extract YouTube video ID from various YouTube URL formats
  const getYoutubeVideoId = (url) => {
    if (!url) return null;
    
    try {
      const urlObj = new URL(url);
      
      if (urlObj.hostname.includes('youtube.com')) {
        return new URLSearchParams(urlObj.search).get('v');
      } else if (urlObj.hostname.includes('youtu.be')) {
        return urlObj.pathname.slice(1);
      }
    } catch (e) {
      return null;
    }
    
    return null;
  };

  // Get YouTube thumbnail URL from video URL
  const getYoutubeThumbnail = (url) => {
    const videoId = getYoutubeVideoId(url);
    return videoId ? `https://img.youtube.com/vi/${videoId}/mqdefault.jpg` : null;
  };

  return (
    <div className="song-submission-screen">
      <header className="screen-header">
        <h1>Füge deine Lieder hinzu</h1>
        <p className="status">
          {isReady 
            ? 'Warte auf andere Spieler...' 
            : `Füge ${lobby?.settings?.songsPerPlayer || 0} Lieder für den Kampf hinzu`}
        </p>
      </header>
      
      <div className="submission-progress">
        <div className="progress-bar">
          <div 
            className="progress-fill" 
            style={{ width: `${(songs.length / (lobby?.settings?.songsPerPlayer || 1)) * 100}%` }}
          ></div>
        </div>
        <p>{songs.length} / {lobby?.settings?.songsPerPlayer || 0} songs</p>
      </div>
      
      <div className="songs-list">
        {songs.map((song, index) => (
          <div key={song.id || index} className="song-card">
            <div className="song-thumbnail">
              {getYoutubeThumbnail(song.youtubeLink) ? (
                <img src={getYoutubeThumbnail(song.youtubeLink)} alt={song.title} />
              ) : (
                <div className="thumbnail-placeholder">
                  <FontAwesomeIcon icon={faMusic} />
                </div>
              )}
            </div>
            <div className="song-info">
              <h3>{song.title}</h3>
              <p>{song.artist}</p>
            </div>
            {!isReady && (
              <button className="btn icon danger" onClick={() => handleRemoveSong(song.id)}>
                <FontAwesomeIcon icon={faTrash} />
              </button>
            )}
          </div>
        ))}
        
        {canSubmitMoreSongs && !isFormVisible && !isReady && (
          <button
            className="add-song-btn"
            onClick={() => setIsFormVisible(true)}
          >
            <FontAwesomeIcon icon={faPlus} />
            <span>Lied hinzufügen</span>
          </button>
        )}
      </div>
      
      {isFormVisible && (
        <div className="song-form-overlay">
          <form className="song-form" onSubmit={handleAddSong}>
            <h3>Lied hinzufügen</h3>
            
            <div className="form-group search-group">
            <label>Lied suchen</label>
            <div className="search-container">
              <input 
                type="text"
                name="searchQuery"
                value={searchQuery}
                onChange={handleInputChange}
                placeholder="YouTube durchsuchen oder Link einfügen..."
                className="search-input"
              />
              {isSearching && (
                <FontAwesomeIcon icon={faSpinner} className="spinner-icon" spin />
              )}
              {searchQuery && !isSearching && (
                <FontAwesomeIcon 
                  icon={faTimes} 
                  className="clear-icon" 
                  onClick={() => {
                    setSearchQuery('');
                    setSearchResults([]);
                    setSelectedVideo(null);
                    setSongForm({ youtubeLink: '' });
                  }} 
                />
              )}
            </div>
            
            {/* Show selected video without embedded player */}
            {selectedVideo && (
              <div className="selected-video">
                <div className="preview-header">
                  <h4>Ausgewähltes Lied</h4>
                  <a 
                    href={selectedVideo.url} 
                    target="_blank" 
                    rel="noopener noreferrer"
                    className="external-link"
                  >
                    <FontAwesomeIcon icon={faExternalLinkAlt} /> Auf YouTube ansehen
                  </a>
                </div>
                
                <div className="video-details">
                  <div className="thumbnail-container">
                    <img 
                      src={selectedVideo.thumbnail} 
                      alt={selectedVideo.title} 
                      className="selected-thumbnail" 
                    />
                    <div className="play-overlay">
                      <FontAwesomeIcon icon={faVideoCamera} />
                    </div>
                  </div>
                  <div className="selected-info">
                    <h4>{selectedVideo.title || 'Unknown Title'}</h4>
                    <p>{selectedVideo.artist || 'Unknown Artist'}</p>
                  </div>
                </div>
              </div>
            )}
            
            {searchResults.length > 0 && (
              <div className="search-results">
                <h4>Suchergebnisse</h4>
                {searchResults.map(result => (
                  <div 
                    key={result.id} 
                    className={`search-result ${selectedVideo && selectedVideo.id === result.id ? 'selected' : ''}`}
                    onClick={() => handleSelectSearchResult(result)}
                  >
                    <div className="result-thumbnail-container">
                      {result.thumbnail ? (
                        <img src={result.thumbnail} alt={result.title} className="result-thumbnail" />
                      ) : (
                        <div className="thumbnail-placeholder">
                          <FontAwesomeIcon icon={faMusic} />
                        </div>
                      )}
                      <div className="thumbnail-overlay">
                        <FontAwesomeIcon icon={faVideoCamera} />
                      </div>
                    </div>
                    <div className="result-info">
                      <h4>{result.title || 'Unknown Title'}</h4>
                      <p>{result.artist || 'Unknown Artist'}</p>
                    </div>
                  </div>
                ))}
              </div>
            )}
          </div>
          
          <div className="form-actions">
            <button type="button" className="btn secondary" onClick={() => {
              setIsFormVisible(false);
              setSearchQuery('');
              setSearchResults([]);
            }}>
              Abbrechen
            </button>
            <button 
              type="submit" 
              className="btn primary"
              disabled={!searchQuery.trim() && !songForm.youtubeLink.trim()}
            >
              Lied hinzufügen
            </button>
          </div>
        </form>
        </div>
      )}
      
      <div className="action-buttons">
        {!isReady && songs.length === lobby?.settings?.songsPerPlayer && (
          <button
            className="btn primary"
            onClick={handleSetReady}
          >
            <FontAwesomeIcon icon={faCheck} /> Kampfbereit
          </button>
        )}
      </div>
      
      {isReady && (
        <div className="waiting-message">
          <h3>Kampfbereit!</h3>
          <p>Warte auf andere Spieler, die ihre Lieder einreichen...</p>
          
          <div className="player-status">
            <h4>Spieler bereit</h4>
            <ul className="players-ready-list">
              {lobby && lobby.players.map(player => (
                <li key={player.id} className={player.isReady ? 'ready' : 'not-ready'}>
                  {player.name} {player.id === currentPlayer.id && '(Du)'} 
                  {player.isReady ? (
                    <FontAwesomeIcon icon={faCheck} className="ready-icon" />
                  ) : (
                    <span className="songs-count">{player.songCount}/{lobby?.settings?.songsPerPlayer}</span>
                  )}
                </li>
              ))}
            </ul>
          </div>
        </div>
      )}
    </div>
  );
};

export default SongSubmissionScreen;
