// VotingScreen.jsx
import { useState, useEffect, useMemo } from 'react';
import { useGame } from '../context/GameContext';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faVoteYea, faTrophy, faMusic, faCheck, faMedal, faCrown } from '@fortawesome/free-solid-svg-icons';
import YouTubeEmbed from './YouTubeEmbed';

function VotingScreen() {
  const { lobby, currentPlayer, submitVote } = useGame();
  const [hasVoted, setHasVoted] = useState(false);
  const [selectedSong, setSelectedSong] = useState(null);
  const [countdown, setCountdown] = useState(null);
  
  // Get current battle
  const battle = lobby?.currentBattle || null;

  // Calculate the tournament phase based on the round number and total songs
  const tournamentPhase = useMemo(() => {
    if (!lobby || !battle) return '';
    
    // Get total number of songs in the tournament
    const totalSongs = lobby.songs?.length || 0;
    
    if (totalSongs === 0) return 'Preliminaries';
    
    // Calculate total rounds needed for the tournament
    const totalRounds = Math.ceil(Math.log2(totalSongs));
    const currentRound = battle.round + 1;
    const roundsRemaining = totalRounds - currentRound;
    
    if (roundsRemaining === 0) return 'Finals';
    if (roundsRemaining === 1) return 'Semi-Finals';
    if (roundsRemaining === 2) return 'Quarter-Finals';
    return 'Contestant Round';
  }, [lobby, battle]);

  // Check if player has already voted
  useEffect(() => {
    if (battle && battle.votes && currentPlayer) {
      // Check if player's ID exists in votes object
      // Since votes is sent as an object, not a Map
      const votesObj = battle.votes || {};
      setHasVoted(Object.prototype.hasOwnProperty.call(votesObj, currentPlayer.id) || 
                  Object.keys(votesObj).includes(currentPlayer.id));
    } else {
      setHasVoted(false);
    }
  }, [battle, currentPlayer]);

  // Handle vote selection
  const handleVoteSelect = (songId) => {
    if (hasVoted) return;
    
    setSelectedSong(songId);
  };

  // Submit final vote
  const handleSubmitVote = async () => {
    if (!selectedSong || hasVoted) return;
    
    await submitVote(selectedSong);
    // Setting hasVoted is now handled by the useEffect that checks votes
  };

  // Get YouTube video IDs from links
  const getYouTubeId = (url) => {
    if (!url) return null;
    
    const regExp = /^.*(youtu.be\/|v\/|u\/\w\/|embed\/|watch\?v=|&v=)([^#&?]*).*/;
    const match = url.match(regExp);
    
    return (match && match[2].length === 11) ? match[2] : null;
  };

  if (!battle || !battle.song1) {
    return (
      <div className="voting-screen">
        <h2>Preparing the next battle...</h2>
      </div>
    );
  }
  
  // Handle "bye" rounds where a song advances automatically
  if (battle.bye === true && battle.song1 && !battle.song2) {
    const song1Id = getYouTubeId(battle.song1?.youtubeLink || '');
    
    return (
      <div className="voting-screen">
        <header className="voting-header">
          <h1>
            <FontAwesomeIcon icon={faTrophy} /> Automatic Advance
          </h1>
          <div className="round-info">
            <span>Round {battle.round + 1}</span>
          </div>
        </header>
        
        <div className="bye-container">
          <div className="song-card bye-winner">
            <div className="song-spotlight"></div>
            <div className="song-details">
              <h3>{battle.song1.title}</h3>
              <p>{battle.song1.artist}</p>
              <div className="auto-advance-notice">
                <p>This song automatically advances to the next round!</p>
              </div>
            </div>
            
            {song1Id ? (
              <div className="video-container">
                <YouTubeEmbed videoId={song1Id} />
              </div>
            ) : (
              <div className="no-video">
                <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
                <span>No video available</span>
              </div>
            )}
          </div>
        </div>
        
        <div className="voting-status">
          <button className="btn primary pixelated full-width" onClick={() => submitVote(battle.song1.id)}>
            Continue to Next Battle
            <span className="pixel-corner tl"></span>
            <span className="pixel-corner tr"></span>
            <span className="pixel-corner bl"></span>
            <span className="pixel-corner br"></span>
          </button>
        </div>
      </div>
    );
  }

  const song1Id = getYouTubeId(battle.song1?.youtubeLink || '');
  const song2Id = getYouTubeId(battle.song2?.youtubeLink || '');

  return (
    <div className="voting-screen">
      <header className="voting-header">
        <h1>
          <FontAwesomeIcon icon={tournamentPhase === 'Finals' ? faCrown : tournamentPhase === 'Semi-Finals' ? faMedal : faVoteYea} /> 
          {tournamentPhase}
        </h1>
        <div className="round-info">
          <span>Round {battle.round + 1}</span>
          {hasVoted && <span className="voted-badge">You voted!</span>}
        </div>
      </header>
      
      <div className="battle-container">
        <div 
          className={`song-card ${selectedSong === battle.song1.id ? 'selected' : ''} ${hasVoted ? 'voted' : ''}`}
          onClick={() => handleVoteSelect(battle.song1.id)}
        >
          <div className="song-spotlight"></div>
          <div className="song-details">
            <h3>{battle.song1.title}</h3>
            <p>{battle.song1.artist}</p>
          </div>
          
          {song1Id ? (
            <div className="video-container">
              <YouTubeEmbed videoId={song1Id} />
              <div className="video-overlay"></div>
            </div>
          ) : (
            <div className="no-video">
              <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
              <span>No video available</span>
            </div>
          )}
          
          {hasVoted && (
            <div className="vote-count">
              <span className="vote-number">{battle.song1Votes}</span>
              <span className="vote-text">votes</span>
            </div>
          )}
          
          {selectedSong === battle.song1.id && !hasVoted && (
            <div className="selection-indicator">
              <FontAwesomeIcon icon={faCheck} className="check-icon" />
            </div>
          )}
        </div>
        
        <div className="versus">
          <span className="versus-text">VS</span>
          <div className="versus-decoration"></div>
        </div>
        
        <div 
          className={`song-card ${selectedSong === battle.song2.id ? 'selected' : ''} ${hasVoted ? 'voted' : ''}`}
          onClick={() => handleVoteSelect(battle.song2.id)}
        >
          <div className="song-spotlight"></div>
          <div className="song-details">
            <h3>{battle.song2.title}</h3>
            <p>{battle.song2.artist}</p>
          </div>
          
          {song2Id ? (
            <div className="video-container">
              <YouTubeEmbed videoId={song2Id} />
              <div className="video-overlay"></div>
            </div>
          ) : (
            <div className="no-video">
              <FontAwesomeIcon icon={faMusic} className="pulse-icon" />
              <span>No video available</span>
            </div>
          )}
          
          {hasVoted && (
            <div className="vote-count">
              <span className="vote-number">{battle.song2Votes}</span>
              <span className="vote-text">votes</span>
            </div>
          )}
          
          {selectedSong === battle.song2.id && !hasVoted && (
            <div className="selection-indicator">
              <FontAwesomeIcon icon={faCheck} className="check-icon" />
            </div>
          )}
        </div>
      </div>
      
      {!hasVoted && (
        <div className="voting-actions">
          <button 
            className="btn primary" 
            onClick={handleSubmitVote}
            disabled={!selectedSong}
          >
            Cast Vote
                      </button>
        </div>
      )}
      
      <div className="voting-status">
        <p>{hasVoted ? 'Waiting for other players to vote...' : 'Choose your favorite!'}</p>
        <div className="votes-count">
          <span>{battle.voteCount || 0}</span> of <span>{lobby?.players?.filter(p => p.isConnected).length || 0}</span> votes
        </div>
        
        {/* Player voting status list */}
        <div className="player-votes">
          <h4>Voters</h4>
          <ul className="players-voted-list">
            {lobby?.players?.filter(p => p.isConnected).map(player => {
              // Check if this player has voted
              const hasPlayerVoted = battle.votes && 
                Object.keys(battle.votes).includes(player.id);
              
              return (
                <li key={player.id} className={hasPlayerVoted ? 'voted' : 'not-voted'}>
                  {player.name} {player.id === currentPlayer.id && '(You)'}
                  {hasPlayerVoted && 
                    <FontAwesomeIcon icon={faCheck} className="vote-icon" />
                  }
                </li>
              );
            })}
          </ul>
        </div>
      </div>
    </div>
  );
}

export default VotingScreen;
