// Socket.IO client instance for real-time communication
import { io } from 'socket.io-client';
import { createContext, useContext, useEffect, useState } from 'react';

// Create a Socket.IO context for use throughout the app
const SocketContext = createContext(null);

// Game context for sharing game state
const GameContext = createContext(null);

export function SocketProvider({ children }) {
  const [socket, setSocket] = useState(null);
  const [isConnected, setIsConnected] = useState(false);

  useEffect(() => {
    // Create socket connection on component mount
    const socketInstance = io(import.meta.env.DEV ? 'http://localhost:5237' : window.location.origin, {
      autoConnect: true,
      reconnection: true,
      reconnectionDelay: 1000,
      reconnectionDelayMax: 5000,
      reconnectionAttempts: 5
    });

    // Socket event listeners
    socketInstance.on('connect', () => {
      setIsConnected(true);
      console.log('Connected to server');
      
      // Try to reconnect to game if we have saved data
      const savedGameData = localStorage.getItem('songBattleGame');
      if (savedGameData) {
        try {
          const { lobbyId, playerName } = JSON.parse(savedGameData);
          if (lobbyId && playerName) {
            console.log(`Attempting to reconnect to lobby: ${lobbyId}`);
            socketInstance.emit('reconnect_to_lobby', { lobbyId, playerName }, (response) => {
              if (response.error) {
                console.error('Reconnection failed:', response.error);
                localStorage.removeItem('songBattleGame');
              } else {
                console.log('Successfully reconnected to lobby');
              }
            });
          }
        } catch (e) {
          console.error('Error parsing saved game data:', e);
          localStorage.removeItem('songBattleGame');
        }
      }
    });

    socketInstance.on('disconnect', () => {
      setIsConnected(false);
      console.log('Disconnected from server');
    });

    socketInstance.on('connect_error', (error) => {
      console.error('Connection error:', error);
    });

    setSocket(socketInstance);

    // Clean up socket connection on unmount
    return () => {
      socketInstance.disconnect();
    };
  }, []);

  return (
    <SocketContext.Provider value={{ socket, isConnected }}>
      {children}
    </SocketContext.Provider>
  );
}

/**
 * Game state provider - manages game state and provides methods for game actions
 */
export function GameProvider({ children }) {
  const { socket, isConnected } = useContext(SocketContext);
  const [lobby, setLobby] = useState(null);
  const [error, setError] = useState(null);
  const [currentPlayer, setCurrentPlayer] = useState(null);

  // Save game info when lobby is joined
  useEffect(() => {
    if (lobby && currentPlayer) {
      sessionStorage.setItem('songBattleGame', JSON.stringify({
        lobbyId: lobby.id,
        playerName: currentPlayer.name
      }));
    }
  }, [lobby, currentPlayer]);

  // Clear error after 5 seconds
  useEffect(() => {
    if (error) {
      const timer = setTimeout(() => {
        setError(null);
      }, 5000);
      return () => clearTimeout(timer);
    }
  }, [error]);

  // Socket event handlers for game updates
  useEffect(() => {
    if (!socket) return;

    // Player joined the lobby
    const handlePlayerJoined = (data) => {
      setLobby(prevLobby => {
        if (!prevLobby) return prevLobby;
        
        // Update the lobby with the new player
        const updatedPlayers = [...prevLobby.players, {
          id: data.playerId,
          name: data.playerName,
          isConnected: true,
          isReady: false,
          songCount: 0
        }];
        
        return {
          ...prevLobby,
          players: updatedPlayers
        };
      });
    };

    // Player reconnected to the lobby
    const handlePlayerReconnected = (data) => {
      setLobby(prevLobby => {
        if (!prevLobby) return prevLobby;
        
        // Update the lobby with the reconnected player
        const updatedPlayers = prevLobby.players.map(player => {
          if (player.name === data.playerName && !player.isConnected) {
            return { ...player, id: data.playerId, isConnected: true };
          }
          return player;
        });
        
        return {
          ...prevLobby,
          players: updatedPlayers
        };
      });
    };

    // Player disconnected from the lobby
    const handlePlayerDisconnected = (data) => {
      setLobby(data.lobby);
    };

    // Game settings were updated
    const handleSettingsUpdated = (data) => {
      setLobby(prevLobby => {
        if (!prevLobby) return prevLobby;
        return {
          ...prevLobby,
          settings: data.settings
        };
      });
    };

    // Game started
    const handleGameStarted = (data) => {
      setLobby(prevLobby => {
        if (!prevLobby) return prevLobby;
        return {
          ...prevLobby,
          state: data.state
        };
      });
    };

    // Songs were updated
    const handleSongsUpdated = (data) => {
      setLobby(data.lobby);
    };

    // Player status changed (ready/not ready)
    const handlePlayerStatusChanged = (data) => {
      console.log('Player status changed, new lobby state:', data.lobby.state);
      setLobby(data.lobby);

      // If the state is VOTING and we have a current battle, explicitly log it
      if (data.lobby.state === 'VOTING' && data.lobby.currentBattle) {
        console.log('Battle ready in player_status_changed:', data.lobby.currentBattle);
      }
    };

    // Vote was submitted
    const handleVoteSubmitted = (data) => {
      console.log('Vote submitted, updating lobby');
      setLobby(data.lobby);
    };

    // New battle started
    const handleNewBattle = (data) => {
      console.log('New battle received:', data.battle);
      setLobby(prevLobby => {
        if (!prevLobby) return prevLobby;
        
        // Ensure we update both the currentBattle and the state
        return {
          ...prevLobby,
          currentBattle: data.battle,
          state: 'VOTING'
        };
      });
    };

    // Game finished
    const handleGameFinished = (data) => {
      setLobby(prevLobby => {
        if (!prevLobby) return prevLobby;
        return {
          ...prevLobby,
          state: 'FINISHED',
          finalWinner: data.winner,
          battles: data.battles
        };
      });
    };

    // Register socket event listeners
    socket.on('player_joined', handlePlayerJoined);
    socket.on('player_reconnected', handlePlayerReconnected);
    socket.on('player_disconnected', handlePlayerDisconnected);
    socket.on('settings_updated', handleSettingsUpdated);
    socket.on('game_started', handleGameStarted);
    socket.on('songs_updated', handleSongsUpdated);
    socket.on('player_status_changed', handlePlayerStatusChanged);
    socket.on('vote_submitted', handleVoteSubmitted);
    socket.on('tournament_started', data => {
      console.log('Tournament started event received:', data);
      setLobby(prevLobby => prevLobby ? {...prevLobby, state: data.state} : prevLobby);
    });
    socket.on('new_battle', handleNewBattle);
    socket.on('game_finished', handleGameFinished);

    // Clean up listeners on unmount
    return () => {
      socket.off('player_joined', handlePlayerJoined);
      socket.off('player_reconnected', handlePlayerReconnected);
      socket.off('player_disconnected', handlePlayerDisconnected);
      socket.off('settings_updated', handleSettingsUpdated);
      socket.off('game_started', handleGameStarted);
      socket.off('songs_updated', handleSongsUpdated);
      socket.off('player_status_changed', handlePlayerStatusChanged);
      socket.off('vote_submitted', handleVoteSubmitted);
      socket.off('new_battle', handleNewBattle);
      socket.off('game_finished', handleGameFinished);
    };
  }, [socket]);

  // Create a lobby
  const createLobby = (playerName) => {
    if (!socket || !isConnected) {
      setError('Not connected to server');
      return;
    }

    socket.emit('create_lobby', { playerName }, (response) => {
      if (response.error) {
        setError(response.error);
      } else {
        setLobby(response.lobby);
        setCurrentPlayer({
          id: socket.id,
          name: playerName,
          isHost: true
        });
      }
    });
  };

  // Join a lobby
  const joinLobby = (lobbyId, playerName) => {
    if (!socket || !isConnected) {
      setError('Not connected to server');
      return;
    }

    socket.emit('join_lobby', { lobbyId, playerName }, (response) => {
      if (response.error) {
        setError(response.error);
      } else {
        setLobby(response.lobby);
        setCurrentPlayer({
          id: socket.id,
          name: playerName,
          isHost: response.lobby.hostId === socket.id
        });
      }
    });
  };

  // Update lobby settings
  const updateSettings = (settings) => {
    if (!socket || !isConnected || !lobby) {
      setError('Not connected to server or no active lobby');
      return;
    }

    socket.emit('update_settings', { settings }, (response) => {
      if (response.error) {
        setError(response.error);
      }
    });
  };

  // Start the game
  const startGame = () => {
    if (!socket || !isConnected || !lobby) {
      setError('Not connected to server or no active lobby');
      return;
    }

    socket.emit('start_game', {}, (response) => {
      if (response.error) {
        setError(response.error);
      }
    });
  };

  // Add a song
  const addSong = (song) => {
    if (!socket || !isConnected || !lobby) {
      setError('Not connected to server or no active lobby');
      return Promise.reject('Not connected to server or no active lobby');
    }

    console.log('Attempting to add song:', song);
    console.log('Current player state:', currentPlayer);
    console.log('Current lobby state before adding song:', lobby);

    return new Promise((resolve, reject) => {
      socket.emit('add_song', { song }, (response) => {
        console.log('Song addition response:', response);
        if (response.error) {
          console.error('Error adding song:', response.error);
          setError(response.error);
          reject(response.error);
        } else if (response.lobby) {
          // Log detailed lobby state for debugging
          console.log('Song added successfully, full lobby response:', response.lobby);
          console.log('Current player songs:', response.lobby.players.find(p => p.id === currentPlayer.id)?.songs);
          console.log('All players song data:', 
            response.lobby.players.map(p => ({
              name: p.name, 
              id: p.id,
              songCount: p.songCount,
              songs: p.songs ? p.songs.map(s => ({id: s.id, title: s.title})) : 'No songs'
            }))
          );
          
          // Force a deep clone of the lobby to ensure React detects the change
          const updatedLobby = JSON.parse(JSON.stringify(response.lobby));
          setLobby(updatedLobby);
          
          // Verify the state was updated correctly
          setTimeout(() => {
            // This won't show the updated state immediately due to React's state update mechanism
            console.log('Lobby state after update (may not reflect immediate changes):', lobby);
            console.log('Updated lobby that was set:', updatedLobby);
          }, 0);
          
          // Resolve with the updated lobby
          resolve(updatedLobby);
        } else {
          console.error('Song addition succeeded but no lobby data was returned');
          setError('Failed to update song list');
          reject('Failed to update song list');
        }
      });
    });
  };
  
  // Search for songs on YouTube
  const searchYouTube = (query) => {
    return new Promise((resolve, reject) => {
      if (!socket || !isConnected) {
        setError('Not connected to server');
        reject('Not connected to server');
        return;
      }

      socket.emit('search_youtube', { query }, (response) => {
        if (response.error) {
          setError(response.error);
          reject(response.error);
        } else {
          resolve(response.results);
        }
      });
    });
  };
  
  // Get metadata for a YouTube video by ID
  const getYouTubeMetadata = (videoId) => {
    return new Promise((resolve, reject) => {
      if (!socket || !isConnected) {
        setError('Not connected to server');
        reject('Not connected to server');
        return;
      }
      
      socket.emit('get_video_metadata', { videoId }, (response) => {
        if (response.error) {
          setError(response.error);
          reject(response.error);
        } else {
          resolve(response.metadata);
        }
      });
    });
  };

  // Remove a song
  const removeSong = (songId) => {
    if (!socket || !isConnected || !lobby) {
      setError('Not connected to server or no active lobby');
      return;
    }

    socket.emit('remove_song', { songId }, (response) => {
      if (response.error) {
        setError(response.error);
      }
    });
  };

  // Set player ready
  const setPlayerReady = () => {
    if (!socket || !isConnected || !lobby) {
      setError('Not connected to server or no active lobby');
      return;
    }

    socket.emit('player_ready', {}, (response) => {
      if (response.error) {
        setError(response.error);
      }
    });
  };

  // Submit a vote
  const submitVote = (songId) => {
    if (!socket || !isConnected || !lobby) {
      setError('Not connected to server or no active lobby');
      return;
    }

    socket.emit('submit_vote', { songId }, (response) => {
      if (response.error) {
        setError(response.error);
      }
    });
  };

  // Leave lobby and clear saved game state
  const leaveLobby = () => {
    localStorage.removeItem('songBattleGame');
    setLobby(null);
    setCurrentPlayer(null);
  };

  return (
    <GameContext.Provider value={{
      lobby,
      error,
      currentPlayer,
      isHost: currentPlayer && lobby && currentPlayer.id === lobby.hostId,
      createLobby,
      joinLobby,
      updateSettings,
      startGame,
      addSong,
      removeSong,
      setPlayerReady,
      submitVote,
      leaveLobby,
      searchYouTube,
      getYouTubeMetadata
    }}>
      {children}
    </GameContext.Provider>
  );
}

// Custom hooks for using contexts
export const useSocket = () => useContext(SocketContext);
export const useGame = () => useContext(GameContext);

// Re-export for better module compatibility
export { SocketContext, GameContext };
